<?php

/**
 * Class PlagScanApi
 * @author Ingmar Szmais <iszmais@databay.de>
 * @since 20.08.19
 */
class API
{

    const HOST = 'https://api.plagscan.com/';
    const VERSION = 'v3';

    /**
     * @var string Base URL
     */
    protected $connection;

    /**
     * @var API
     */
    protected static $instance;

    /**
     * @return API
     */
    public static function getInstance() : API
    {
        if (self::$instance === null) {
            self::$instance = new self();
        }
        return self::$instance;
    }

    /**
     * PlagScanApi constructor.
     */
    public function __construct()
    {
        $this->connection = self::HOST . self::VERSION;
    }

    /**
     * Make a HTTP request to the API
     *
     * @param string $endPoint
     * @param string $requestType
     * @param string $accessToken
     * @param array $data
     *
     * @return array|string
     */
    public function request(string $endPoint, string $requestType, string $accessToken = '', array $data = [])
    {

        $ch = curl_init();

        $url = $this->connection . '/' . $endPoint;

        if ($accessToken) {
            $url .= '?access_token=' . $accessToken;
        }

        if ($requestType === 'GET' || $requestType === 'PATCH') {
            foreach ($data as $key => $value) {
                $url .= '&' . $key . '=' . $value;
            }
            if (strpos($url, '?') === false) {
                $url = preg_replace('/\&/', '?', $url, 1);
            }
            $data = [];
        }

        $curlopt = [
            CURLOPT_URL => $url,
            CURLOPT_RETURNTRANSFER => 1,
            CURLOPT_CUSTOMREQUEST => $requestType,
        ];

        if (!empty($data)) {
            $curlopt = [CURLOPT_POSTFIELDS => $data] + $curlopt;
        }
        curl_setopt_array($ch, $curlopt);

        $response = curl_exec($ch);
        curl_close($ch);

        $response = json_decode($response, true) ?? $response;
        if (isset($response['data'])) {
            return $response['data'];
        }
        if (isset( $response['error'])) {
            return $response['error']['message'];
        }
        return $response;
    }
}